@@ -4,13 +4,17 @@ import "fmt"
 
 func fn() {
 LABEL:
-	if true {
-		fmt.Println("")
-		fmt.Println("")
-		fmt.Println("")
-		fmt.Println("")
-		break // want `missing whitespace above this line \(too many lines above branch\)`
+	for range 1 {
+		if true {
+			fmt.Println("")
+			fmt.Println("")
+			fmt.Println("")
+			fmt.Println("")
+			break // want `missing whitespace above this line \(too many lines above branch\)`
+		}
 	}
+
+	goto LABEL
 }
 
 func fn1() {
@@ -47,13 +51,21 @@ L2: // want `missing whitespace above this line \(never cuddle label\)`
 	_ = 1
 L3: // want `missing whitespace above this line \(never cuddle label\)`
 	_ = 1
+
+	goto L1
+
+	goto L2
+
+	goto L3
 }
 
-func fn4() {
+func fn4(err error) {
 	if true { // want +1 `unnecessary whitespace \(leading-whitespace\)`
 
 	LABEL:
 		panic(err)
+
+		goto LABEL
 	}
 }
 
@@ -62,6 +74,10 @@ func fn5() {
 LABEL: // want `missing whitespace above this line \(never cuddle label\)`
 	if true {
 	}
+
+	goto LABEL
+
+	_ = a
 }
 
 func fn6() {
@@ -71,4 +87,8 @@ ADD: // want `missing whitespace above this line \(never cuddle label\)`
 	i++
 SUB: // want `missing whitespace above this line \(never cuddle label\)`
 	i--
+
+	goto ADD
+
+	goto SUB
 }
